import os
import pwd
import shutil
import logging
#import common.libs.lib_config
from subprocess import Popen, PIPE

LOG_FILENAME = '/var/log/userful-scripts.log'
LOG_FORMAT = \
    '"%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(lineno)d:"%(message)s'
BACKUP_SUFFIX = '.backup'
REMOVE_SUFFIX = '.remove'

def import_file(module_name, file_name):
    '''
    Make a new module object from the code in specified file
    '''
    from types import ModuleType
    module = ModuleType(module_name)
    module_file = open(file_name, 'r')
    exec module_file in module.__dict__
    return module

class config(object):
    def __init__(self, file_name = None):
        if not file_name:
            file_name = '/etc/userful/path.conf'
        path_file = open(file_name, 'r')
        try:
            lines = path_file.readlines()
        except:
            return None
        finally:
            path_file.close()

        for line in lines:
            splitted = line.split('=')
            if splitted > 1:
                key = splitted[0]
                key = key.strip()
                value = ''.join(splitted[1:])
                value = value.strip()
                self.__dict__[key] = value

def logger(name, file_name = None):
    if not file_name:
        file_name = LOG_FILENAME

    logging.basicConfig(filename = LOG_FILENAME, datefmt = '%Y-%m-%d %H:%M:%S',
                        format = LOG_FORMAT, level = logging.DEBUG)
    return logging.getLogger(name)

def get_home_dirs():
    collected = []
    for user_info in pwd.getpwall():
        home_dir = user_info.pw_dir
        if not os.path.exists(home_dir + os.sep + '.config'):
            continue # skip special users
        collected.append(home_dir)
    return collected

def get_user_by_dir(home_dir):
    for user_info in pwd.getpwall():
        if home_dir == user_info.pw_dir:
            return user_info.pw_name
    return None

def locate_binary(tool):
    result = None
    stdout, __stderr = Popen("which %s" % tool, stdout = PIPE,
                   stderr = PIPE, shell = True).communicate()
    if len(stdout) > 0:
        result = unicode(stdout).splitlines()[0]
    return result


def backup(file_name):
    file_copy = os.path.join(config().backup_dir,  file_id(file_name))
    if os.path.exists(file_copy + BACKUP_SUFFIX):
        return True
    if os.path.exists(file_name): # backup user's file
        shutil.copy(file_name, file_copy + BACKUP_SUFFIX)
    else:
        open(file_name + REMOVE_SUFFIX, "w").close()
    return True

def restore(file_name):
    file_copy = os.path.join(config().backup_dir,  file_id(file_name))
    if os.path.exists(file_copy + BACKUP_SUFFIX):
        shutil.move(file_copy + BACKUP_SUFFIX, file_name)
    if os.path.exists(file_copy + REMOVE_SUFFIX):
        os.remove(file_copy + REMOVE_SUFFIX)
        if os.path.exists(file_name):
            os.remove(file_name)

def file_id(full_path_to_file):
    import hashlib
    return hashlib.md5(full_path_to_file).hexdigest()

def ensure_dir(file_name):
    if os.path.isdir(file_name):
        dir_name = file_name
    else:
        dir_name = os.path.dirname(file_name)
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)

def deploy(src, dst):
    backup(dst)
    shutil.copy(src, dst)

def gconf_set(profile_name, key, value):
    cmd = ['userful-bridge', '--gconf', 'set', '("%s", "%s", %s)'
           % (profile_name, key, value)]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    __stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return True

def call(cmd, user, **kwargs):
    '''
    Call method for integration points
    '''
    bridge_cmd = ['userful-bridge', '--misc', 'call',
                  """('%s', '%s')""" % (cmd, user), """%s""" % str(kwargs)]
    proc = Popen(bridge_cmd)
    __stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return True

def create_mime_type(mime_type, glob_pattern):
    cmd = ['userful-bridge', '--mime', 'create', '("%s", "%s")'
           % (mime_type, glob_pattern)]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    __stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return True

def assign_mime_type(mime_type, name, Exec, **kwargs):
    cmd = ['userful-bridge', '--mime', 'assign', '("%s", "%s", "%s") %s'
           % (mime_type, name, Exec, str(kwargs))]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    __stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return True

def install_from_spec(home_dir, spec_file):
    user_name = get_user_by_dir(home_dir)
    if not user_name:
        return False
    cmd = ['userful-bridge', '--install', 'spec', '("%s", "%s", "%s")'
           % (user_name, home_dir, spec_file)]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    __stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return True

def is_admin(user_name):
    cmd = ['userful-bridge', '--user', 'is_admin', '("%s")' % (user_name)]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return stdout == 'True'

def is_valid_user(user_name):
    cmd = ['userful-bridge', '--user', 'is_user', '("%s")' % (user_name)]
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    stdout, stderr = proc.communicate()
    if stderr != '':
        return False
    return stdout == 'True'

def get_valid_users():
    """
    Returns a list of all valid users that have been created on the
    system.  The each user's login name is a string in the list.  If
    there's no users on the system, then an empty string is returned.
    """
    cmd = ['userful-bridge', '--user', 'get_valid_users']
    proc = Popen(cmd, stdout = PIPE, stderr = PIPE)
    stdout, stderr = proc.communicate()
    if stderr != '':
        return []
    return stdout.split()
