#!/bin/bash

# Copyright (c) 2005-2012 Userful Corporation. All rights reserved.
# http://www.userful.com/
#

UCC_CONF=/etc/userful/ucc.conf
PORT=9000
if [ -r $UCC_CONF ]; then
	source $UCC_CONF
	if [ -n $ucc_port ]; then
		PORT=$ucc_port
	fi
fi

source $UCC_CONF
PORT=$ucc_port

PATH_CONF=/etc/userful/path.conf
source $PATH_CONF

LOCALE=`$sbin_dir/userful-i18n -l`

DISTRO=`/usr/sbin/userful-bridge --misc what_distro`
case "${DISTRO}" in
  UBUNTU* )
        ADMIN_GROUPS="admin adm sudo root"
        CONTROL_CENTER_APP="firefox"
        ;;
  BOSS* | USERFUL6 )
        ADMIN_GROUPS=""
        CONTROL_CENTER_APP="xulrunner"
        ;;
    * )
        ADMIN_GROUPS=""
        CONTROL_CENTER_APP="firefox"
        ;;
esac

CONTROL_CENTER_WRAPPER="${CONTROL_CENTER_APP} -app ${userful_dir}/frontend/control-center/wrapper-xul/application.ini"
CONTROLCENTER_URL="http://localhost:${PORT}/html/ControlCenter.html?locale=$LOCALE"

WEB_SERVER_PID="/var/run/userful/userful-webserver.pid"
FRONDEND_PROXY_PID="/var/run/userful/userful-frontendproxy.pid"

is_running () {
	# Given a pid file, and check if the corresongding proc is running.
	if [ ! -e "$1" ]; then
	    # If pid file doesn't exist, then return false
		return 1
	fi
	pid=`cat $1`
	test -e /proc/$pid
	return $?
}

if [ "$1" == "--reportaproblem" ] ; then
    # No authentication,
    # Show a basic control center with no administrative features
    $CONTROL_CENTER_WRAPPER "$CONTROLCENTER_URL&report_problem=1" > /dev/null 2>&1 &
    exit 0
fi

open_limited_ucc () {
    # If the current logged in user has no admin privilege,

    # Check if web server and frondend proxy are running
    is_running $WEB_SERVER_PID
    is_web_server_running=$?

    is_running $FRONDEND_PROXY_PID
    is_frontend_proxy_running=$?

    if [ "$is_web_server_running" -eq "0" ] && [ "$is_frontend_proxy_running" -eq "0" ]; then
	    # Show a basic control center with no administrative features
	    $CONTROL_CENTER_WRAPPER "$CONTROLCENTER_URL&no_auth=1" > /dev/null 2>&1 &
	    return 0
	fi

	return 1
}

if [[ $UID -eq 0 ]]; then
  # current user is root
  ROOTUSER=1
  # It's admin, UCC should run in full version
  is_admin="yes"
else
  echo Run as user
  ROOTUSER=0
  # Check if the user is in the admin group
  if [ -n "$ADMIN_GROUPS" ]; then
	user_groups=`groups`
	for user_group in $user_groups
	do
		for admin_group in $ADMIN_GROUPS
		do
			#echo "$user_group ==== $admin_group"
			if [ "$user_group" == "$admin_group" ]; then
				is_admin="yes"
			fi
		done
	done
  else
    # If no admin group defined for the distro, every users should be considered as admins
    is_admin="yes"
  fi
fi


if [ "$is_admin" != "yes" ]; then
	echo "open limited function UCC"
	open_limited_ucc
	if [ $? == 0 ]; then
		exit 0
	fi
fi

SU="pkexec"

export PATH=$PATH:.:/usr/share/userful/bin


if [[ $ROOTUSER -eq 1 ]]; then
    SESSION_INFO=`$userful_dir/frontend/start-control-center-backend.sh $DISPLAY`
else
    SESSION_INFO=`$SU $userful_dir/frontend/start-control-center-backend.sh $DISPLAY`
fi

RET=$?

if [ $RET != 0 ] ; then
    echo "Get session ID script returns error: " $RET
    # echo "session info: " $SESSION_INFO

	echo "open limited function UCC"
	open_limited_ucc
	if [ $? != 0 ]; then
		echo "Can't open UCC because the frondend proxy or web.py server is not running."
		#$sbin_dir/userful-message-display ControlCenterCantGetRootAccess
	fi
    exit 0
fi

STN_ID=`echo $SESSION_INFO | cut -d ' ' -f 1`
AUTH_TOKEN=`echo $SESSION_INFO | cut -d ' ' -f 2`

if [ -z "$AUTH_TOKEN" ] ; then
    echo "Starting Control Center without authentication token"
    if [ -z "$STN_ID" ] ; then
        $CONTROL_CENTER_WRAPPER "$CONTROLCENTER_URL"
    else
        $CONTROL_CENTER_WRAPPER "$CONTROLCENTER_URL&station_id=$STN_ID"
    fi

    exit 0
fi

echo "Starting Control Center"
$CONTROL_CENTER_WRAPPER "$CONTROLCENTER_URL&auth_token=$AUTH_TOKEN&station_id=$STN_ID"

exit 0
